<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Datacube OGC Health Check</title>
</head>
<body>
    <h1>PONG!!</h1>
    <p>Database: {{ status }}</p>
    <ul>
        {% for env, status in statuses.items() %}
            <li>
                {{ env }}:
                {% if status %}UP{% else %}DOWN{% endif %}
            </li>
        {% endfor %}
    </ul>
</body>
</html>
